package com.diagbot.facade;

import com.diagbot.client.StandConvertServiceClient;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.StandConvert;
import io.github.lvyahui8.spring.facade.DataFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 提示信息facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class IndicationFacade {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    BillFacade billFacade;
    @Autowired
    StandConvertServiceClient standConvertServiceClient;

    private static final Map<String, List<String>> methodMap;

    static {
        methodMap = new HashMap<>();
        methodMap.put("1", Arrays.asList("crf", "stand"));
        methodMap.put("2", Arrays.asList("crf", "stand"));
        methodMap.put("3", Arrays.asList("crf", "stand"));
        methodMap.put("4", Arrays.asList("crf", "stand"));
    }


    /**
     * 病情提示总入口
     *
     * @param
     * @return
     */
    public IndicationDTO indicationFac(IndicationPushVO indicationPushVO) {
        IndicationDTO res = new IndicationDTO();
        Map<String, Object> debug = new LinkedHashMap<>();
        //方法适配处理
        List<String> ruleTypeList = Arrays.asList(indicationPushVO.getRuleType().split(","));
        indicationPushVO.setRuleTypeList(ruleTypeList);

        List<String> methodList = new ArrayList<>();
        for (String ruleType : ruleTypeList) {
            if (ListUtil.isNotEmpty(methodMap.get(ruleType))) {
                List<String> methods = methodMap.get(ruleType);
                for (String method : methods) {
                    if (!methodList.contains(method)) {
                        methodList.add(method);
                    }
                }
            }
        }

        WordCrfDTO wordCrfDTO = new WordCrfDTO();
        long l1 = System.currentTimeMillis();

        // 模型处理数据
        long crfStart = System.currentTimeMillis();
        if (methodList.contains("crf")) {
            wordCrfDTO = commonFacade.crf_process(indicationPushVO);
        }
        CoreUtil.getDebugStr(crfStart, "模型处理耗时", debug);

        // 标准词转换
        long standStart = System.currentTimeMillis();
        if (methodList.contains("stand")) {
            StandConvert standConvert = commonFacade.dataTypeGet(wordCrfDTO);
            Map<String, Map<String, String>> standConvertMap = neoFacade.standConvertCrf(standConvert);
            commonFacade.dataTypeSet(wordCrfDTO, standConvertMap);
        }
        CoreUtil.getDebugStr(standStart, "标准词转换耗时", debug);

        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("wordCrfDTO", wordCrfDTO);
            invokeParams.put("indicationPushVO", indicationPushVO);
            invokeParams.put("debug", debug);
            res = DataFacade.get("indicationAll", invokeParams, IndicationDTO.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "提醒类服务出错" + e.getMessage());
        }

        // 输入调试信息
        CoreUtil.getDebugStr(l1, "本次调用总计耗时", res.getDebug());
        CoreUtil.getDebugObject("数据", res.getDebug(), wordCrfDTO);
        return res;
    }
}
