package com.diagbot.facade;

import com.diagbot.dto.DiseaseInfoDTO;
import com.diagbot.dto.DrugInfoDTO;
import com.diagbot.dto.LisDetailDTO;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.entity.node.LisName;
import com.diagbot.entity.node.LisSet;
import com.diagbot.entity.node.Medicine;
import com.diagbot.entity.node.PacsName;
import com.diagbot.entity.node.YiBaoDiseaseName;
import com.diagbot.entity.node.YiBaoOperationName;
import com.diagbot.enums.StaticTypeEnum;
import com.diagbot.repository.*;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/12 16:44
 */
@Component
public class RetrievalFacade {

    @Autowired
    LisSetRepository lisSetRepository;

    @Autowired
    LisNameRepository lisRepository;

    @Autowired
    YiBaoDiseaseRepository icdDiseaseRepository;

    @Autowired
    PacsNameRepository pacsRepository;

    @Autowired
    MedicineRepository medicineRepository;

    @Autowired
    YiBaoOperationNameRepository operationRepository;

    @Autowired
    TransfusionRemindRepository transfusionRemindRepository;

    @Autowired
    BaseNodeRepository baseNodeRepository;

    @Autowired
    DeptRepository deptRepository;

    /**
     * 检索
     *
     * @param retrievalVO
     * @return
     */
    public RetrievalDTO index(RetrievalVO retrievalVO) {
        RetrievalDTO retrievalDTO = new RetrievalDTO();
        retrievalVO.setInputStr(retrievalVO.getInputStr().trim().toUpperCase());
        switch (retrievalVO.getType()) {
            case 1:
                retrievalDTO.setLisNames(getLisPacks(retrievalVO.getInputStr()));
                break;
            case 2:
                retrievalDTO.setLisDetailNames(getLis(retrievalVO.getInputStr()));
                break;
            case 3:
                retrievalDTO.setPacsNames(getPACSs(retrievalVO.getInputStr()));
                break;
            case 4:
                retrievalDTO.setDiseaseNames(getDiseases(retrievalVO.getInputStr()));
                break;
            case 5:
                retrievalDTO.setDrugNames(getMedicines(retrievalVO.getInputStr()));
                break;
            case 6:
                retrievalDTO.setOperationNames(getOperations(retrievalVO.getInputStr()));
                break;
            case 7:
                retrievalDTO.setDeptNames(getDepts(retrievalVO.getInputStr()));
                break;
            case 8:
                retrievalDTO.setTransfusionNames(getTransfusion(retrievalVO.getInputStr()));
                break;
        }

        return retrievalDTO;
    }


    /*
     * 获取化验套餐名称列表
     */
    public List<String> getLisPacks(String info) {

        LisSetNode lisPackNode = new LisSetNode();
        List<String> lisPacklist = lisPackNode.getLISPackNameList(info, lisSetRepository, baseNodeRepository);

        return lisPacklist;
    }


    /*
     * 获取化验细项名称列表
     */
    public List<LisDetailDTO> getLis(String info) {

        LisNameNode lisNode = new LisNameNode();
        List<LisDetailDTO> lislist = lisNode.getLisDetailList(info, lisRepository, baseNodeRepository);

        return lislist;
    }


    /*
     * 获取辅检名称列表
     */
    public List<String> getPACSs(String info) {

        PacsNameNode pacsNode = new PacsNameNode();
        List<String> pacslist = pacsNode.getPACSNameList(info, pacsRepository, baseNodeRepository);

        return pacslist;
    }


    /*
     * 获取疾病名称列表
     */
    public List<DiseaseInfoDTO> getDiseases(String info) {

        YiBaoDiseaseNode yiBaoDiseaseNode = new YiBaoDiseaseNode();
        List<DiseaseInfoDTO> diseaselist = yiBaoDiseaseNode.getDiseaseNameList(info, icdDiseaseRepository, baseNodeRepository);

        return diseaselist;
    }


    /*
     * 获取药品商品名称列表
     */
    public List<DrugInfoDTO> getMedicines(String info) {

        MedicineNode medicineNode = new MedicineNode();
        List<DrugInfoDTO> medslist = medicineNode.getMedicineList(info, medicineRepository, baseNodeRepository);

        return medslist;
    }


    /*
     * 获取手术和操作名称列表
     */
    public List<OperationInfoDTO> getOperations(String info) {

        YiBaoOperationNameNode operationNode = new YiBaoOperationNameNode();
        List<OperationInfoDTO> opslist = operationNode.getOperationList(info, operationRepository, baseNodeRepository);

        return opslist;
    }


    /*
     * 获取输血方式列表
     */
    public List<String> getTransfusion(String info) {
        List<String> transfusionNames = transfusionRemindRepository.findTransfusionByName(info);

        return transfusionNames;
    }


    /*
     * 获取科室名称列表
     */
    public List<String> getDepts(String name) {

        DeptNode deptNode = new DeptNode();
        List<String> deptlist = deptNode.getDeptNameList(name, deptRepository, baseNodeRepository);

        return deptlist;
    }


    /**
     * 医学知识（静态信息）检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();
        String inputStr = staticKnowledgeIndexVO.getInputStr().trim().toUpperCase();
        List<Integer> types = staticKnowledgeIndexVO.getTypes();

        //TODO 诊断
        List<YiBaoDiseaseName> icdDiseaseList = icdDiseaseRepository.findByNameContaining(inputStr);
        List<StaticKnowledgeIndexDTO> disRetList = Lists.newLinkedList();
        for (YiBaoDiseaseName icdDisease : icdDiseaseList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(icdDisease.getName());
            staticDTO.setType(StaticTypeEnum.DIS.getKey());
            staticDTO.setTypeName(StaticTypeEnum.DIS.getName());
            disRetList.add(staticDTO);
        }
        //TODO 药品
        List<Medicine> medicineList = medicineRepository.findByNameContaining(inputStr);

        List<StaticKnowledgeIndexDTO> drugRetList = Lists.newLinkedList();
        for (Medicine medicine : medicineList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(medicine.getName());
            staticDTO.setType(StaticTypeEnum.DRUG.getKey());
            staticDTO.setTypeName(StaticTypeEnum.DRUG.getName());
            drugRetList.add(staticDTO);
        }
        //TODO 检验套餐
        List<LisSet> lisSetList = lisSetRepository.findByNameContaining(inputStr);

        List<StaticKnowledgeIndexDTO> lisSetRetList = Lists.newLinkedList();
        for (LisSet lisSet : lisSetList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(lisSet.getName());
            staticDTO.setType(StaticTypeEnum.LISSET.getKey());
            staticDTO.setTypeName(StaticTypeEnum.LISSET.getName());
            lisSetRetList.add(staticDTO);
        }
        //TODO 检验明细关联出检验套餐
        List<LisName> lisList = lisRepository.findByNameContaining(inputStr);

        List<StaticKnowledgeIndexDTO> lisSetRelatecLisRetList = Lists.newLinkedList();
        for (LisName lis : lisList) {
            if (lis.getLissets().size() > 0) {
                for (LisSet lisSet : lis.getLissets()) {
                    StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                    staticDTO.setRetrievalName(lis.getName());
                    staticDTO.setName(lisSet.getName());
                    staticDTO.setType(StaticTypeEnum.LISSET.getKey());
                    staticDTO.setTypeName(StaticTypeEnum.LISSET.getName());
                    lisSetRelatecLisRetList.add(staticDTO);
                }
            }
        }
        //合并检验套餐
        Map<String, List<StaticKnowledgeIndexDTO>> lisSetMap
                = EntityUtil.makeEntityListMap(lisSetRetList, "name");
        Map<String, List<StaticKnowledgeIndexDTO>> lisSetRelatecLisMap
                = EntityUtil.makeEntityListMap(lisSetRelatecLisRetList, "name");
        for (Map.Entry<String, List<StaticKnowledgeIndexDTO>> entry : lisSetRelatecLisMap.entrySet()) {
            if (lisSetMap == null
                    || lisSetMap.size() == 0
                    || !lisSetMap.containsKey(entry.getKey())) {
                lisSetRetList.add(entry.getValue().get(0));
            }
        }
        //TODO 检验明细
        lisList = lisRepository.findByNameContaining(inputStr);

        List<StaticKnowledgeIndexDTO> lisRetList = Lists.newLinkedList();
        for (LisName lis : lisList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(lis.getName());
            staticDTO.setType(StaticTypeEnum.LIS.getKey());
            staticDTO.setTypeName(StaticTypeEnum.LIS.getName());
            lisRetList.add(staticDTO);
        }
        //TODO 检查
        List<PacsName> pacsList = pacsRepository.findByNameContaining(inputStr);

        List<StaticKnowledgeIndexDTO> pacsRetList = Lists.newLinkedList();
        for (PacsName pacs : pacsList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(pacs.getName());
            staticDTO.setType(StaticTypeEnum.PACS.getKey());
            staticDTO.setTypeName(StaticTypeEnum.PACS.getName());
            pacsRetList.add(staticDTO);
        }


        //TODO 手术和操作
        List<YiBaoOperationName> operationList = operationRepository.findByNameContaining(inputStr);

        List<StaticKnowledgeIndexDTO> operationRetList = Lists.newLinkedList();

        for (YiBaoOperationName operation : operationList) {
            StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
            staticDTO.setName(operation.getName());
            staticDTO.setType(StaticTypeEnum.OPT.getKey());
            staticDTO.setTypeName(StaticTypeEnum.OPT.getName());
            operationRetList.add(staticDTO);
        }

        //0-全部、1-诊断、2-药品、3-检验套餐、4-检验明细、5-检查、6-手术和操作
        if (ListUtil.isEmpty(types) || types.contains(0)) {
            retList.addAll(disRetList);
            retList.addAll(drugRetList);
            retList.addAll(lisSetRetList);
            //retList.addAll(lisSetRelatecLisRetList);
            retList.addAll(lisRetList);
            retList.addAll(pacsRetList);
            retList.addAll(operationRetList);
        } else {
            if (types.contains(1)) {
                retList.addAll(disRetList);
            }
            if (types.contains(2)) {
                retList.addAll(drugRetList);
            }
            if (types.contains(3)) {
                //套餐
                retList.addAll(lisSetRetList);
                //明细关联出的套餐
                //retList.addAll(lisSetRelatecLisRetList);
                //明细
                retList.addAll(lisRetList);
            }
            if (types.contains(5)) {
                retList.addAll(pacsRetList);
            }
            if (types.contains(6)) {
                retList.addAll(operationRetList);
            }
        }
        return retList;
    }
}