package com.diagbot.facade;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.client.StandConvertServiceClient;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.StandConvertCrfBatchDTO;
import com.diagbot.dto.StandConvertCrfDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.Drug;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.ItemExt;
import com.diagbot.vo.StandConvertCrfVO;
import com.diagbot.vo.TestAllVO;
import com.diagbot.vo.TestIndicationVO;
import com.diagbot.vo.TestLineVO;
import io.github.lvyahui8.spring.facade.DataFacade;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * @Description: 测试facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class TestFacade {

    @Autowired
    IndicationFacade indicationFacade;
    @Autowired
    StandConvertServiceClient standConvertServiceClient;
    @Autowired
    RedisUtil redisUtil;

    /**
     * 开单项数据测试
     *
     * @param file
     */
    public Map<String, Object> importExcel(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("2");
            indicationPushVO.setIdNum(bean.getIdNum());
            if (StringUtil.isNotBlank(bean.getOrderType())) {
                switch (bean.getOrderType()) {
                    case "实验室检查名称": // 细项
                        List<Lis> lisDetailOrder = new ArrayList<>();
                        Lis lisDetailBean = new Lis();
                        lisDetailBean.setName(bean.getStandName());
                        lisDetailBean.setDetailName(bean.getStandName());
                        lisDetailBean.setUniqueName(bean.getStandName());
                        lisDetailOrder.add(lisDetailBean);
                        indicationPushVO.setLisOrder(lisDetailOrder);
                        break;
                    case "辅助检查名称":
                        List<Pacs> pacsOrder = new ArrayList<>();
                        Pacs pacsBean = new Pacs();
                        pacsBean.setName(bean.getStandName());
                        pacsBean.setUniqueName(bean.getStandName());
                        pacsOrder.add(pacsBean);
                        indicationPushVO.setPacsOrder(pacsOrder);
                        break;
                    case "实验室检查套餐名":
                        List<Lis> lisOrder = new ArrayList<>();
                        Lis lisBean = new Lis();
                        lisBean.setName(bean.getStandName());
                        lisBean.setUniqueName(bean.getStandName());
                        lisOrder.add(lisBean);
                        indicationPushVO.setLisOrder(lisOrder);
                        break;
                    case "医保手术和操作名称":
                        List<Item> operationOrder = new ArrayList<>();
                        Item operationItem = new Item();
                        operationItem.setName(bean.getStandName());
                        operationItem.setUniqueName(bean.getStandName());
                        operationOrder.add(operationItem);
                        indicationPushVO.setOperationOrder(operationOrder);
                        break;
                    case "药品通用名称":
                        List<Drug> drugOrder = new ArrayList<>();
                        Drug drug = new Drug();
                        drug.setName(bean.getStandName());
                        drug.setUniqueName(bean.getStandName());
                        drugOrder.add(drug);
                        indicationPushVO.setDrugOrder(drugOrder);
                        break;
                    default:
                        continue;
                }
            }

            indicationPushVO.setMsg(bean.getNeoName());

            if (StringUtil.isNotEmpty(bean.getNeoType())) {
                switch (bean.getNeoType()) {
                    case "1": // 性别
                        if ("男性".equals(bean.getNeoName())) {
                            indicationPushVO.setSex(1);
                        } else if ("女性".equals(bean.getNeoName())) {
                            indicationPushVO.setSex(2);
                        }
                        break;
                    case "2": // 化验, 示例：血小板计数，＜，50，10^9/L
                        String lisStr = bean.getNeoName();
                        String[] lisArr = lisStr.split("，");

                        List<Lis> lisList = new ArrayList<>();
                        Lis lisBean = new Lis();
                        lisBean.setUniqueName(lisArr[0]);
                        lisBean.setName(lisArr[0]);
                        lisBean.setDetailName(lisArr[0]);
                        lisBean.setValue(getValue(lisArr));
                        lisList.add(lisBean);
                        indicationPushVO.setLis(lisList);
                        break;
                    case "3": // 临床表现
                        indicationPushVO.setChief(bean.getNeoName());
                        break;
                    case "4": //辅助检查描述
                        indicationPushVO.setSymptom("B超:提示" + bean.getNeoName());
                        break;
                    case "5": // 诊断
                        List<Item> diag = new ArrayList<>();
                        Item diagItem = new Item();
                        diagItem.setName(bean.getNeoName());
                        diagItem.setUniqueName(bean.getNeoName());
                        diag.add(diagItem);
                        indicationPushVO.setDiag(diag);
                        break;
                    case "6": // 服用药品
                        StringBuffer symptom = new StringBuffer("发病来，神志清，精神可，胃纳一般，睡眠可，二便无殊。一直服用");
                        symptom.append(bean.getNeoName()).append("。");
                        indicationPushVO.setSymptom(symptom.toString());
                        break;
                    case "10": // 禁忌人群
                        if ("妊娠".equals(bean.getNeoName())) {
                            List<Item> groupList = new ArrayList<>();
                            Item group = new Item();
                            group.setName(bean.getNeoName());
                            group.setUniqueName(bean.getNeoName());
                            groupList.add(group);
                            indicationPushVO.setDiag(groupList);
                        } else {
                            continue;
                        }
                    case "7" : // 药品过敏原
                        StringBuffer pastStr = new StringBuffer("");
                        if ("药品通用名称".equals(bean.getDrugType())) {
                            pastStr.append("有“").append(bean.getNeoName()).append("”过敏。");
                            indicationPushVO.setPasts(pastStr.toString());
                        } else if ("药品类别".equals(bean.getDrugType())) {
                            List<String> drugList = (List<String>)redisUtil.get("药品分类_" + bean.getNeoName());
                            if (ListUtil.isNotEmpty(drugList)) {
                                if (testLineVO.isGetOne()) {
                                    pastStr.append("有“").append(drugList.get(getRandomNum(drugList.size()))).append("”过敏。");
                                    indicationPushVO.setPasts(pastStr.toString());
                                } else {
                                    for (String s : drugList) {
                                        pastStr = new StringBuffer("");
                                        IndicationPushVO indicationPushCopy = new IndicationPushVO();
                                        BeanUtil.copyProperties(indicationPushVO, indicationPushCopy);
                                        pastStr.append("有“").append(s).append("”过敏。");
                                        indicationPushCopy.setPasts(pastStr.toString());
                                        indicationPushCopy.setMsg("过敏药品【" + s + "】");
                                        indicationPushVOList.add(indicationPushCopy);
                                    }
                                    continue;
                                }
                            }
                        }
                        break;
                    case "12": // 体征
                        StringBuffer sbVital = new StringBuffer();
                        String[] vitalArr = bean.getNeoName().split("，");
                        if (vitalArr.length > 1) {
                            if ("收缩压".equals(vitalArr[0]) || "舒张压".equals(vitalArr[0])) {
                                if ("收缩压".equals(vitalArr[0])) {
                                    sbVital.append("血压");
                                    sbVital.append(getValue(vitalArr) + "/100mmHg");
                                    indicationPushVO.setVital(sbVital.toString());
                                } else if ("舒张压".equals(vitalArr[0])) {
                                    sbVital.append("血压");
                                    sbVital.append("100/" + getValue(vitalArr) + "mmHg");
                                    indicationPushVO.setVital(sbVital.toString());
                                }
                            } else {
                                // 其他数值类型
                                sbVital.append(vitalArr[0]);
                                sbVital.append(getValue(vitalArr));
                                sbVital.append(vitalArr[3]);
                                indicationPushVO.setVital(sbVital.toString());
                            }
                        } else {
                            sbVital.append(vitalArr[0]);
                            indicationPushVO.setVital(sbVital.toString());
                        }
                        break;
                    case "14": // 年龄
                        String[] splitAge = bean.getNeoName().split(",");
                        indicationPushVO.setAge((int) getValueNum(splitAge));
                        break;
                    default:
                        continue;
                }
            }
            indicationPushVOList.add(indicationPushVO);
        }

        // Map<String, Object> map = getDebugMap(indicationPushVOList, "2", start);
        Map<String, Object> map = getDebugMapMulti(indicationPushVOList, "2", start);
        return map;
    }

    /**
     * 开单项数据测试——输血
     *
     * @param file
     */
    public Map<String, Object> testIndicationTransfusion(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            if (StringUtil.isBlank(bean.getTransfusionNodeName())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("2");
            indicationPushVO.setIdNum(bean.getIdNum());
            if (StringUtil.isNotBlank(bean.getTransfusionName())) {
                List<ItemExt> transfusionOrder = new ArrayList<>();
                ItemExt item = new ItemExt();
                item.setName(bean.getTransfusionName());
                item.setUniqueName(bean.getTransfusionName());
                transfusionOrder.add(item);
                indicationPushVO.setTransfusionOrder(transfusionOrder);
            }

            if (StringUtil.isNotEmpty(bean.getTransfusionNodeName())) {
                switch (bean.getTransfusionNodeName()) {
                    case "手术和操作": // 手术和操作
                        List<Item> operation = new ArrayList<>();
                        Item item = new Item();
                        item.setName(bean.getStandName());
                        item.setUniqueName(bean.getStandName());
                        operation.add(item);
                        indicationPushVO.setOperation(operation);
                        break;
                    case "医保疾病名称": // 医保疾病名称
                        List<Item> diag = new ArrayList<>();
                        Item itemDiag = new Item();
                        itemDiag.setName(bean.getStandName());
                        itemDiag.setUniqueName(bean.getStandName());
                        diag.add(itemDiag);
                        indicationPushVO.setDiag(diag);
                        break;
                    case "实验室检查": // 实验室检查
                        List<Lis> lisList = new ArrayList<>();
                        Lis lisBean = new Lis();
                        lisBean.setUniqueName(bean.getStandName());
                        lisBean.setName(bean.getStandName());
                        lisBean.setDetailName(bean.getStandName());
                        lisBean.setValue(getValue(bean.getSymbol(), bean.getValue()));
                        lisList.add(lisBean);
                        indicationPushVO.setLis(lisList);
                        break;
                    default:
                        continue;
                }
            }
            indicationPushVOList.add(indicationPushVO);
        }

        Map<String, Object> map = getDebugMap(indicationPushVOList, "2", start);
        return map;
    }

    /**
     * 高危手术测试API
     *
     * @param file
     */
    public Map<String, Object> testHighRiskOperationExcel(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            // 3,4是高危手术
            if (StringUtil.isBlank(bean.getOperationLevel()) || "0".equals(bean.getOperationLevel()) || "1".equals(bean.getOperationLevel()) || "2".equals(bean.getOperationLevel())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("3");
            indicationPushVO.setIdNum(bean.getIdNum());
            List<Item> operationOrder = new ArrayList<>();
            Item item = new Item();
            item.setUniqueName(bean.getOperationName());
            item.setName(bean.getOperationName());
            operationOrder.add(item);
            indicationPushVO.setOperationOrder(operationOrder);
            indicationPushVO.setMsg(bean.getOperationName());
            indicationPushVOList.add(indicationPushVO);
        }

        Map<String, Object> map = getDebugMap(indicationPushVOList, "3", start);
        return map;
    }

    /**
     * 其他值提醒测试——化验
     *
     * @param file
     */
    public Map<String, Object> testOtherTipLis(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("4");
            indicationPushVO.setIdNum(bean.getIdNum());

            if ("药品通用名称".equals(bean.getOtherTipNodeType())) {
                List<Drug> drug = new ArrayList<>();
                Drug item = new Drug();
                item.setUniqueName(bean.getOtherTipNodeName());
                item.setName(bean.getOtherTipNodeName());
                drug.add(item);
                indicationPushVO.setDrug(drug);
            } else if ("医保疾病名称".equals(bean.getOtherTipNodeType())) {
                List<Item> diag = new ArrayList<>();
                Item item = new Item();
                item.setUniqueName(bean.getOtherTipNodeName());
                item.setName(bean.getOtherTipNodeName());
                diag.add(item);
                indicationPushVO.setDiag(diag);
            }
            List<Lis> lisList = new ArrayList<>();
            Lis lis = new Lis();
            lis.setName("套餐");
            lis.setDetailName("明细");
            lis.setUniqueName(bean.getOtherTipLisName());
            lis.setValue(getValue(bean.getSymbol(), bean.getValue()));
            lisList.add(lis);
            indicationPushVO.setLis(lisList);
            indicationPushVOList.add(indicationPushVO);
        }

        Map<String, Object> map = getDebugMap(indicationPushVOList, "4", start);
        return map;
    }

    /**
     * 其他值提醒测试——辅检
     *
     * @param file
     */
    public Map<String, Object> testOtherTipPacs(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("4");
            indicationPushVO.setIdNum(bean.getIdNum());

            if ("男性".equals(bean.getSexStr())) {
                indicationPushVO.setSex(1);
            } else if ("女性".equals(bean.getSexStr())) {
                indicationPushVO.setSex(2);
            }
            if (StringUtil.isNotBlank(bean.getDisName())) {
                List<Item> diag = new ArrayList<>();
                Item item = new Item();
                item.setUniqueName(bean.getDisName());
                item.setName(bean.getDisName());
                diag.add(item);
                indicationPushVO.setDiag(diag);
            }
            if (bean.getAgeRange() != null) {
                if (bean.getAgeRange() == 0) {
                    if (bean.getAgeMin() != null) {
                        indicationPushVO.setAge(bean.getAgeMin() + 1);
                    } else if (bean.getAgeMax() != null) {
                        indicationPushVO.setAge(bean.getAgeMin() - 1);
                    }
                } else if (bean.getAgeRange() == 1) {
                    if (bean.getAgeMin() != null) {
                        indicationPushVO.setAge(bean.getAgeMin() - 1);
                    } else if (bean.getAgeMax() != null) {
                        indicationPushVO.setAge(bean.getAgeMax() + 1);
                    }
                }
            }
            if (StringUtil.isNotBlank(bean.getPacsDesc())) {
                Pacs pacs = new Pacs();
                pacs.setName("胸部CT");
                pacs.setUniqueName("胸部CT");
                pacs.setResult(bean.getPacsDesc());
                indicationPushVO.getPacs().add(pacs);
            }

            if (StringUtil.isNotBlank(bean.getPacsDescHas())) {
                Pacs pacs = new Pacs();
                pacs.setName("CT");
                pacs.setUniqueName("CT");
                pacs.setResult(bean.getPacsDescHas());
                indicationPushVO.getPacs().add(pacs);
            }

            indicationPushVOList.add(indicationPushVO);
        }

        Map<String, Object> map = getDebugMap(indicationPushVOList, "4", start);
        return map;
    }

    /**
     * 其他值提醒测试——输血
     *
     * @param file
     */
    public Map<String, Object> testOtherTipTransfusion(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("4");
            indicationPushVO.setIdNum(bean.getIdNum());

            if (StringUtil.isNotBlank(bean.getDisName())) {
                List<Item> diag = new ArrayList<>();
                Item item = new Item();
                item.setUniqueName(bean.getDisName());
                item.setName(bean.getDisName());
                diag.add(item);
                indicationPushVO.setDiag(diag);
            }
            if (StringUtil.isNotBlank(bean.getOperationName())) {
                List<Item> operation = new ArrayList<>();
                Item item = new Item();
                item.setUniqueName(bean.getOperationName());
                item.setName(bean.getOperationName());
                operation.add(item);
                indicationPushVO.setOperation(operation);
            }
            if ("实验室检查名称".equals(bean.getItemNodeName())) {
                List<Lis> lisList = new ArrayList<>();
                Lis lis = new Lis();
                lis.setName("套餐");
                lis.setDetailName("明细");
                lis.setUniqueName(bean.getItemName());
                lis.setValue(getValue(bean.getSymbol(), bean.getValue()));
                lisList.add(lis);
                indicationPushVO.setLis(lisList);
                indicationPushVOList.add(indicationPushVO);
            }
        }

        Map<String, Object> map = getDebugMap(indicationPushVOList, "4", start);
        return map;
    }

    /**
     * 高危药品测试API
     *
     * @param file
     */
    public Map<String, Object> testHighRiskDrugExcel(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            if (StringUtil.isEmpty(bean.getDrugLevel())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("3");
            indicationPushVO.setIdNum(bean.getIdNum());
            List<Drug> drugOrder = new ArrayList<>();
            Drug item = new Drug();
            item.setUniqueName(bean.getDrugHighRisk());
            item.setName(bean.getDrugHighRisk());
            item.setForm(bean.getDrugForm());
            drugOrder.add(item);
            indicationPushVO.setDrugOrder(drugOrder);
            indicationPushVO.setMsg(bean.getDrugHighRisk() + bean.getDrugForm());
            indicationPushVOList.add(indicationPushVO);
        }

        Map<String, Object> map = getDebugMap(indicationPushVOList, "3", start);
        return map;
    }

    /**
     * 危急值数据测试——化验
     *
     * @param file
     */
    public Map<String, Object> importCriticalLisExcel(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("1");
            if (StringUtil.isNotBlank(bean.getCriticalAge()) && "成人".equals(bean.getCriticalAge())) {
                indicationPushVO.setAge(20);
            } else if (StringUtil.isNotBlank(bean.getCriticalAge()) && "新生儿".equals(bean.getCriticalAge())) {
                indicationPushVO.setAge(1);
            } else {
                indicationPushVO.setAge(20);
            }
            indicationPushVO.setSex(1);
            indicationPushVO.setIdNum(bean.getIdNum());

            List<Lis> lisDetail = new ArrayList<>();
            if (bean.getCriticalMin() != null) {
                Lis lisDetailBean = new Lis();
                lisDetailBean.setName(bean.getCriticalMeal());
                lisDetailBean.setDetailName(bean.getCriticalLis());
                lisDetailBean.setUniqueName(bean.getCriticalLis());
                lisDetailBean.setValue(bean.getCriticalMin());
                lisDetail.add(lisDetailBean);
            } else if (bean.getCriticalMax() != null) {
                Lis lisDetailBean = new Lis();
                lisDetailBean.setName(bean.getCriticalMeal());
                lisDetailBean.setDetailName(bean.getCriticalLis());
                lisDetailBean.setUniqueName(bean.getCriticalLis());
                lisDetailBean.setValue(bean.getCriticalMax());
                lisDetail.add(lisDetailBean);
            }
            indicationPushVO.setLis(lisDetail);
            indicationPushVO.setMsg(bean.getCriticalLis());

            indicationPushVOList.add(indicationPushVO);
        }

        Map<String, Object> map = getDebugMap(indicationPushVOList, "1", start);
        return map;
    }

    /**
     * 危急值数据测试——辅检
     *
     * @param file
     */
    public Map<String, Object> importCriticalPacsExcel(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            indicationPushVO.setRuleType("1");

            List<Pacs> pacsList = new ArrayList<>();
            Pacs pacs = new Pacs();
            pacs.setName("腹部B超");
            pacs.setUniqueName("腹部B超");
            pacs.setResult(bean.getRuleText());
            pacsList.add(pacs);
            indicationPushVO.setPacs(pacsList);
            indicationPushVO.setMsg(bean.getRuleText());
            indicationPushVO.setIdNum(bean.getIdNum());

            indicationPushVOList.add(indicationPushVO);
        }

        Map<String, Object> map = getDebugMap(indicationPushVOList, "1", start);
        return map;
    }

    /**
     * 返回数值
     *
     * @param arr
     * @return
     */
    public double getValue(String[] arr) {
        if ("＜".equals(arr[1]) || "≤".equals(arr[1]) || "<".equals(arr[1]) || "≤".equals(arr[1])) {
            return Double.parseDouble(arr[2]) - 0.1;
        } else if ("＞".equals(arr[1]) || "≥".equals(arr[1]) || (">").equals(arr[1])) {
            return Double.parseDouble(arr[2]) + 0.1;
        }
        return 0.0;
    }

    /**
     * 返回数值
     *
     * @param arr
     * @return
     */
    public double getValueNum(String[] arr) {
        if ("<".equals(arr[1]) || "＜".equals(arr[1]) || "≤".equals(arr[1])) {
            return Double.parseDouble(arr[2]) - 1;
        } else if ("＞".equals(arr[1]) || "≥".equals(arr[1]) || ">".equals(arr[1])) {
            return Double.parseDouble(arr[2]) + 1;
        }
        return 0.0;
    }

    /**
     * 返回数值
     *
     * @param dou
     * @return
     */
    public double getValue(String symbol, Double dou) {
        if ("＜".equals(symbol) || "≤".equals(symbol)) {
            return dou - 0.1;
        } else if ("＞".equals(symbol) || "≥".equals(symbol)) {
            return dou + 0.1;
        }
        return 0.0;
    }

    /**
     * 标准词转换
     *
     * @param standConvertCrfVO
     * @return
     */
    public StandConvertCrfDTO testStandConvert(StandConvertCrfVO standConvertCrfVO) {
        StandConvertCrfDTO standConvertCrfDTO = standConvertServiceClient.similarity(standConvertCrfVO);
        return standConvertCrfDTO;
    }

    /**
     * 标准词转换(批量)
     *
     * @param standConvertCrfVOList
     * @return
     */
    public StandConvertCrfBatchDTO testStandConvertBatch(List<StandConvertCrfVO> standConvertCrfVOList) {
        StandConvertCrfBatchDTO standConvertCrfBatchDTO = standConvertServiceClient.similarityBatch(standConvertCrfVOList);
        return standConvertCrfBatchDTO;
    }

    /**
     * 复杂高危手术测试API
     *
     * @param file
     * @param testLineVO
     * @return
     */
    public Map<String, Object> testHighRiskProcess(MultipartFile file, TestLineVO testLineVO) {
        long start = System.currentTimeMillis();
        List<IndicationPushVO> indicationPushVOList = new ArrayList<>();
        List<TestIndicationVO> data = ExcelUtils.importExcelMultiSheets(file, 0, 1, testLineVO.getSheetIndex(), TestIndicationVO.class);
        for (TestIndicationVO bean : data) {
            if (StringUtil.isNotEmpty(testLineVO.getIdNum()) && !testLineVO.getIdNum().equals(bean.getIdNum())) {
                continue;
            }
            // 只留下等级为2的数据
            if ("0".equals(bean.getOperationLevel()) || "1".equals(bean.getOperationLevel())
                    || "3".equals(bean.getOperationLevel()) || "4".equals(bean.getOperationLevel())) {
                continue;
            }
            String operationNameMsg = bean.getOperationName();

         /*    String operationName = bean.getOperationName();
            List<Item> operation = new ArrayList<>();
            String[] operationStrArray = operationName.split("，");
            for(String str :operationStrArray){
                Item item = new Item();
                item.setName(str);
                item.setUniqueName(str);
                operation.add(item);
            }
            indicationPushVO.setOperation(operation);*/
           //年龄
            IndicationPushVO indicationPushVOAge = new IndicationPushVO();
            String[] split = bean.getAgeAll().split(",");
            setname(indicationPushVOAge,bean);
            indicationPushVOAge.setAge((int) getValueNum(split));
            indicationPushVOAge.setMsg("年龄："+(int) getValueNum(split)+operationNameMsg);
            indicationPushVOAge.setRuleType("3");
            indicationPushVOAge.setIdNum(bean.getIdNum());
            indicationPushVOList.add(indicationPushVOAge);

           //体格
            String physique = bean.getPhysique();
            String[] physiqueStrArray = physique.split("，");
            for (int i = 0;i<physiqueStrArray.length;i++) {
                String str = physiqueStrArray[i];
                if(testLineVO.isGetOne()){
                    int sum = getRandomNum(physiqueStrArray.length);
                    str =physiqueStrArray[sum];
                }
                IndicationPushVO indphysiquePushVO = new IndicationPushVO();
                setname(indphysiquePushVO,bean);
                StringBuffer sbVital = new StringBuffer();
                String[] vitalArr = str.split(",");
                if (vitalArr.length > 1) {
                    if ("收缩压".equals(vitalArr[0].trim()) || "舒张压".equals(vitalArr[0].trim())) {
                        if ("收缩压".equals(vitalArr[0].trim())) {
                            sbVital.append("血压");
                            sbVital.append(getValue(vitalArr) + "/100mmHg");
                            indphysiquePushVO.setIdNum(bean.getIdNum());
                            indphysiquePushVO.setRuleType("3");
                            indphysiquePushVO.setMsg("体格：" + sbVital+"=="+operationNameMsg);
                            indphysiquePushVO.setVital(sbVital.toString());
                            indicationPushVOList.add(indphysiquePushVO);
                        } else if ("舒张压".equals(vitalArr[0])) {

                            sbVital.append("血压");
                            sbVital.append("100/" + getValue(vitalArr) + "mmHg");
                            indphysiquePushVO.setIdNum(bean.getIdNum());
                            indphysiquePushVO.setRuleType("3");
                            indphysiquePushVO.setMsg("体格：" + sbVital+"=="+operationNameMsg);
                            indphysiquePushVO.setVital(sbVital.toString());
                            indicationPushVOList.add(indphysiquePushVO);
                        }
                    } else {
                        // 其他数值类型
                        sbVital.append(vitalArr[0]);
                        sbVital.append(getValue(vitalArr));
                        sbVital.append(vitalArr[3]);
                        indphysiquePushVO.setIdNum(bean.getIdNum());
                        indphysiquePushVO.setRuleType("3");
                        indphysiquePushVO.setMsg("体格：" + sbVital+"=="+operationNameMsg);
                        indphysiquePushVO.setVital(sbVital.toString());
                        indicationPushVOList.add(indphysiquePushVO);
                    }
                } else {
                    sbVital.append(vitalArr[0]);
                    indphysiquePushVO.setIdNum(bean.getIdNum());
                    indphysiquePushVO.setRuleType("3");
                    indphysiquePushVO.setMsg("体格：" + sbVital+"=="+operationNameMsg);
                    indphysiquePushVO.setVital(sbVital.toString());
                    indicationPushVOList.add(indphysiquePushVO);
                }
                if(testLineVO.isGetOne()){
                    break;
                }
            }

           //化验
            String lisAll = bean.getLisAll();
            String[] physiquelisAll = lisAll.split("，");
            for (int i = 0;i<physiquelisAll.length;i++) {

                String str = physiquelisAll[i];
                if(testLineVO.isGetOne()){
                    int sum = getRandomNum(physiquelisAll.length);
                    str =physiquelisAll[sum];
                }
                IndicationPushVO indicationPushVOlisList = new IndicationPushVO();
                setname(indicationPushVOlisList,bean);
                List<Lis> lisList = new ArrayList<>();
                if (str.contains(",")) {
                    //包含英文逗号
                    String[] strArray = str.split(",");
                    if(strArray.length>2) {
                        Lis lisBean = new Lis();
                        lisBean.setUniqueName(strArray[0].trim());
                        lisBean.setName(strArray[0].trim());
                        lisBean.setDetailName(strArray[0].trim());
                        lisBean.setValue(getValue(strArray));
                        lisList.add(lisBean);
                        indicationPushVOlisList.setIdNum(bean.getIdNum());
                        indicationPushVOlisList.setRuleType("3");
                        indicationPushVOlisList.setMsg("化验" + strArray[0].trim() + "==" + operationNameMsg);
                        indicationPushVOlisList.setLis(lisList);
                        indicationPushVOList.add(indicationPushVOlisList);
                    }else {
                        Lis lisBean = new Lis();
                        lisBean.setUniqueName(strArray[0].trim());
                        lisBean.setName(strArray[0]);
                        lisBean.setDetailName(strArray[0].trim());
                        lisBean.setOtherValue(strArray[1].trim());
                        lisList.add(lisBean);
                        indicationPushVOlisList.setIdNum(bean.getIdNum());
                        indicationPushVOlisList.setRuleType("3");
                        indicationPushVOlisList.setMsg("化验：" + strArray[0].trim() + "==" + operationNameMsg);
                        indicationPushVOlisList.setLis(lisList);
                        indicationPushVOList.add(indicationPushVOlisList);
                    }
                }
                if(testLineVO.isGetOne()){
                    break;
                }
            }

            //辅检
            String pacsAll = bean.getPacsAll();
            String[] physiquepacsAll = pacsAll.split("，");
            //indicationPushVO.setSymptom(pacsAll);

            for (int i = 0;i<physiquepacsAll.length;i++) {

                String pacsStr = physiquepacsAll[i];
                if(testLineVO.isGetOne()){
                    int sum = getRandomNum(physiquepacsAll.length);
                    pacsStr =physiquepacsAll[sum];
                }
                IndicationPushVO indicationPushVOpacs = new IndicationPushVO();
                setname(indicationPushVOpacs,bean);
                List<Pacs> pacs = new ArrayList<>();
                Pacs pacs1 = new Pacs();
                pacs1.setUniqueName("腹部CT");
                pacs1.setName("腹部CT");
                pacs1.setResult(pacsStr.trim());
                pacs.add(pacs1);
                indicationPushVOpacs.setIdNum(bean.getIdNum());
                indicationPushVOpacs.setRuleType("3");
                indicationPushVOpacs.setMsg("辅检："+pacsStr+"=="+operationNameMsg);
                indicationPushVOpacs.setPacs(pacs);
                indicationPushVOpacs.setSymptom(pacsAll);
                indicationPushVOList.add(indicationPushVOpacs);
                if(testLineVO.isGetOne()){
                    break;
                }
            }

           //诊断"腹部CT"
            String disease = bean.getDisease();
            String[] physiquedisease = disease.split("，");

            for (int i = 0;i<physiquedisease.length;i++) {

                String diseaseStr = physiquedisease[i];
                if(testLineVO.isGetOne()){
                    int sum = getRandomNum(physiquelisAll.length);
                    diseaseStr =physiquedisease[sum];
                }
                IndicationPushVO indicationPushVOpacs = new IndicationPushVO();
                setname(indicationPushVOpacs,bean);
                List<Item> diag = new ArrayList<>();
                Item item = new Item();
                item.setUniqueName(diseaseStr.trim());
                item.setName(diseaseStr.trim());
                diag.add(item);
                indicationPushVOpacs.setMsg("诊断："+diseaseStr+"=="+operationNameMsg);
                indicationPushVOpacs.setDiag(diag);
                indicationPushVOpacs.setIdNum(bean.getIdNum());
                indicationPushVOpacs.setRuleType("3");
                indicationPushVOList.add(indicationPushVOpacs);
                if(testLineVO.isGetOne()){
                    break;
                }
            }
            //List<Item> operationOrder = new ArrayList<>();
            // setname(indicationPushVO,bean);
            //  indicationPushVOList.add(indicationPushVO);
        }

        Map<String, Object> map = getDebugMapMulti(indicationPushVOList, "3", start);
        return map;
    }

    /**
     * 提示方法
     *
     * @param indicationPushVOList
     * @param ruleType
     * @return
     */
    public Map getDebugMap(List<IndicationPushVO> indicationPushVOList, String ruleType, long start) {
        Map<String, Object> map = new LinkedHashMap<>();
        List<String> msg = new ArrayList<>();
        List<String> errMsg = new ArrayList<>();
        for (IndicationPushVO indicationPushVO : indicationPushVOList) {
            try {
                IndicationDTO indicationDTO = indicationFacade.indicationFac(indicationPushVO);
                switch (ruleType) {
                    case "1":
                        if (ListUtil.isEmpty(indicationDTO.getCriticalValList())) {
                            msg.add("第【" + indicationPushVO.getIdNum() + "】行未匹配，"
                                    + (StringUtil.isNotBlank(indicationPushVO.getMsg()) ? indicationPushVO.getMsg() : ""));
                        }
                        break;
                    case "2":
                        if (ListUtil.isEmpty(indicationDTO.getBillMsgList())) {
                            msg.add("第【" + indicationPushVO.getIdNum() + "】行未匹配。"
                                    + (StringUtil.isNotBlank(indicationPushVO.getMsg()) ? indicationPushVO.getMsg() : ""));
                        }
                        break;
                    case "3":
                        if (ListUtil.isEmpty(indicationDTO.getHighRiskList())) {
                            msg.add("第【" + indicationPushVO.getIdNum() + "】行未匹配。"
                                    + (StringUtil.isNotBlank(indicationPushVO.getMsg()) ? indicationPushVO.getMsg() : ""));
                        }
                        break;
                    case "4":
                        if (ListUtil.isEmpty(indicationDTO.getOtherList())) {
                            msg.add("第【" + indicationPushVO.getIdNum() + "】行未匹配。"
                                    + (StringUtil.isNotBlank(indicationPushVO.getMsg()) ? indicationPushVO.getMsg() : ""));
                        }
                        break;
                    default:
                        throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "ruleType格式不对");
                }
            } catch (Exception e) {
                System.out.println(e.getMessage());
                errMsg.add(indicationPushVO.getIdNum() + "行出错了");
            }
        }
        map.put("总条数", indicationPushVOList.size() + "条");
        map.put("出错条数", msg.size() + "条");
        map.put("出错信息", msg);
        map.put("程序报错", errMsg);
        map.put("执行时间", (System.currentTimeMillis() - start) / 1000.0 + "秒");
        return map;
    }

    /**
     * 多线程执行
     *
     * @param indicationPushVOList
     * @param start
     * @return
     */
    public Map getDebugMapMulti(List<IndicationPushVO> indicationPushVOList, String ruleType, long start) {
        Map<String, Object> map = new LinkedHashMap<>();
        List<String> errMsg = new ArrayList<>();
        try {
            int threadNum = 8; // 推荐的总线程数 = cpu个数 * 每个cpu的核数
            List<List<IndicationPushVO>> threadData = divideByCopies(indicationPushVOList, threadNum);
            List<Future<Map>> list = new ArrayList<>();

            ExecutorService executor = Executors.newFixedThreadPool(threadNum);
            for (int i = 0; i < threadNum; i++) {
                final int index = i; // 这一行代码很重要，如果使用成员变量，下面会报错
                Future<Map> future = executor.submit(new Callable<Map>() {
                    @Override
                    public Map call() {
                        List<IndicationPushVO> data = threadData.get(index);
                        return getDebugMap(data, ruleType, start);
                    }
                });
                list.add(future);
            }

            // 拼接返回数据
            for (int i = 0; i < list.size(); i++) {
                Future<Map> future = list.get(i);
                while(!future.isDone()); // 这一行代码很重要
                List<String> obj = (List<String>)future.get().get("出错信息");
                if (ListUtil.isNotEmpty(obj)) {
                    errMsg.addAll(obj);
                }
            }
            map.put("总条数", indicationPushVOList.size() + "条");
            map.put("出错信息", errMsg);
            map.put("出错条数", errMsg.size() + "条");
            Long t2 = System.currentTimeMillis();
            map.put("执行时间", (t2 - start) / 1000.0 + "秒");
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "多线程执行高风险手术复杂情况失败:" + e.getMessage());
        }
        return map;
    }

    public void setname(IndicationPushVO indicationPushVO, TestIndicationVO bean) {
        List<Item> operationOrder = new ArrayList<>();
        Item item = new Item();
        item.setUniqueName(bean.getOperationName());
        item.setName(bean.getOperationName());
        operationOrder.add(item);
        indicationPushVO.setOperationOrder(operationOrder);
    }

    public Map testAll(TestAllVO testAllVO) {
        if (StringUtil.isBlank(testAllVO.getPath())) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "参数为空，未填写【文件所在路径】");
        }
        long start = System.currentTimeMillis();
        String path = testAllVO.getPath();
        Map<String, Object> map = new LinkedHashMap<>();
        Map<String, Object> paramMap = new LinkedHashMap<>();
        Map<String, String> mapPath = new LinkedHashMap<>();
        mapPath.put("【危急值_化验】", path + File.separator + "【危急值_化验】.xlsx");
        mapPath.put("【危急值_辅检】", path + File.separator + "【危急值_辅检】.xls");
        mapPath.put("【其他值提醒_化验】", path + File.separator + "【其他值提醒_化验】.xlsx");
        mapPath.put("【其他值提醒_辅检】", path + File.separator + "【其他值提醒_辅检】.xlsx");
        mapPath.put("【其他值提醒_输血】", path + File.separator + "【其他值提醒_输血】.xlsx");
        mapPath.put("【高危药品】", path + File.separator + "【高危药品】.xlsx");
        mapPath.put("【高危手术】", path + File.separator + "【高危手术】.xlsx");
        mapPath.put("【开单合理性】", path + File.separator + "【开单合理性】.xlsx");
        mapPath.put("【开单合理性_输血】", path + File.separator + "【开单合理性_输血】.xlsx");
        paramMap.put("mapPath", mapPath);

        try {
            map = DataFacade.get("testAll", paramMap, Map.class);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "提醒类服务出错" + e.getMessage());
        }

        map.put("总耗时", (System.currentTimeMillis() - start) / 1000.0 + "秒");
        return map;
    }

    public MultipartFile getMulFileByPath(String picPath) {
        FileItem fileItem = createFileItem(picPath);
        MultipartFile mfile = new CommonsMultipartFile(fileItem);
        return mfile;
    }

    public FileItem createFileItem(String filePath) {
        FileItemFactory factory = new DiskFileItemFactory(16, null);
        String textFieldName = "textField";
        int num = filePath.lastIndexOf(".");
        String extFile = filePath.substring(num);
        FileItem item = factory.createItem(textFieldName, "text/plain", true,
                "MyFileName" + extFile);
        File newfile = new File(filePath);
        int bytesRead = 0;
        byte[] buffer = new byte[8192];
        try
        {
            FileInputStream fis = new FileInputStream(newfile);
            OutputStream os = item.getOutputStream();
            while ((bytesRead = fis.read(buffer, 0, 8192))
                    != -1)
            {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            fis.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return item;
    }

    /**
     * 随机获取0—size之间的一个数值
     * @param size
     * @return
     */
    public int getRandomNum(int size) {
        return (int)(Math.random() * size);
    }

    /**
     * 将列表按列表总长度划分
     *
     * @param originList 数据
     * @param num 份数
     * @param <T>
     * @return
     */
    public <T> List<List<T>> divideByCopies(List<T> originList, int num) {
        List<List<T>> list = new ArrayList<>();
        if (ListUtil.isEmpty(originList) || num < 0) {
            return null;
        }
        for (int i = 0; i < num; i++) {
            list.add(new ArrayList<T>());
        }
        for (int i = 0; i < originList.size(); i++) {
            list.get(i % num).add(originList.get(i));
        }

        return list;
    }
}
