package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.NeoEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.facade.NeoFacade;
import com.diagbot.model.entity.Clinical;
import com.diagbot.model.entity.Medicine;
import com.diagbot.model.entity.Operation;
import com.diagbot.model.label.ChiefLabel;
import com.diagbot.model.label.DiagLabel;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.PresentLabel;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.DrugRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.rule.SexRule;
import com.diagbot.rule.VitalRule;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.Drug;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 开单总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillProcess {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    DrugRule drugRule;
    @Autowired
    SexRule sexRule;
    @Autowired
    AgeRule ageRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;
    @Autowired
    VitalRule vitalRule;
    @Autowired
    GroupRule groupRule;


    public void process(List<BillNeoDTO> billNeoDTOList, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 合并图谱数据到同一个对象中
        List<BillNeoMaxDTO> billNeoMaxDTOList = new ArrayList<>();
        for (BillNeoDTO billNeoDTO : billNeoDTOList) {
            BillNeoMaxDTO billNeoMaxDTO =  new BillNeoMaxDTO();
            if (billNeoDTO.getDrugBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getDrugBillNeoDTO(), billNeoMaxDTO);
                billNeoMaxDTO.setType(TypeEnum.drug.getName());
            } else if (billNeoDTO.getLisBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getLisBillNeoDTO(), billNeoMaxDTO);
                billNeoMaxDTO.setType(TypeEnum.lis.getName());
            } else if (billNeoDTO.getPacsBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getPacsBillNeoDTO(), billNeoMaxDTO);
                billNeoMaxDTO.setType(TypeEnum.pacs.getName());
            } else if (billNeoDTO.getOperationBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getOperationBillNeoDTO(), billNeoMaxDTO);
                if (ListUtil.isNotEmpty(billNeoDTO.getOperationBillNeoDTO().getMedallegen())) {
                    billNeoMaxDTO.getAllergicmeds().addAll(billNeoDTO.getOperationBillNeoDTO().getMedallegen());
                }
                billNeoMaxDTO.setType(TypeEnum.operation.getName());
            } else if (billNeoDTO.getTransfusionBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getTransfusionBillNeoDTO(), billNeoMaxDTO);
                billNeoMaxDTO.setType(TypeEnum.transfusion.getName());
            }
            billNeoMaxDTO.setOrderName(billNeoDTO.getName()); // 开单名称
            billNeoMaxDTO.setOrderStandName(billNeoDTO.getStandname()); // 开单标准名称
            // TODO 测试数据开始
            // if (billNeoDTO.getName().equals("普通胃镜检查")) {
            //     NodeNeoDTO sexNeo = new NodeNeoDTO();
            //     sexNeo.setName("男性");
            //     billNeoMaxDTO.setGender(sexNeo);
            //
            //     NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            //     nodeNeoDTO.setName("红细胞压积");
            //     nodeNeoDTO.setVal("阳性");
            //     billNeoDTO.getPacsBillNeoDTO().getLis().add(nodeNeoDTO);
            //
            //     NodeNeoDTO nodepacsDTO = new NodeNeoDTO();
            //     nodepacsDTO.setName("上腹部平扫");
            //     billNeoDTO.getPacsBillNeoDTO().getPacs().add(nodepacsDTO);
            //
            //     NodeNeoDTO opereat = new NodeNeoDTO();
            //     opereat.setName("胸腔镜下左下肺叶切除术");
            //     billNeoDTO.getPacsBillNeoDTO().getOperations().add(opereat);
            //     NodeNeoDTO opereat2 = new NodeNeoDTO();
            //     opereat2.setName("区域淋巴结清扫术");
            //     billNeoDTO.getPacsBillNeoDTO().getOperations().add(opereat2);
            //
            //     NodeNeoDTO drug = new NodeNeoDTO();
            //     drug.setName("青霉素类");
            //     billNeoDTO.getPacsBillNeoDTO().getAllergicmeds().add(drug);
            //
            //     NodeNeoDTO drug3 = new NodeNeoDTO();
            //     drug3.setName("泰舒达类");
            //     billNeoDTO.getPacsBillNeoDTO().getOralmeds().add(drug3);
            //
            //     // NodeNeoDTO vital = new NodeNeoDTO();
            //     // vital.setName("体温");
            //     // vital.setMax(new BigDecimal("39.1"));
            //     // vital.setMin(new BigDecimal("35.1"));
            //     // vital.setValType(1);
            //     // billNeoDTO.getPacsBillNeoDTO().getVitals().add(vital);
            //
            //     NodeNeoDTO vitalBp = new NodeNeoDTO();
            //     vitalBp.setName("舒张压");
            //     vitalBp.setMax(new BigDecimal("120"));
            //     vitalBp.setMin(new BigDecimal("100"));
            //     vitalBp.setValType(1);
            //     billNeoDTO.getPacsBillNeoDTO().getVitals().add(vitalBp);
            //
            //     NodeNeoDTO vitalBp2 = new NodeNeoDTO();
            //     vitalBp2.setName("收缩压");
            //     vitalBp2.setMax(new BigDecimal("120"));
            //     vitalBp2.setMin(new BigDecimal("100"));
            //     vitalBp2.setValType(1);
            //     billNeoDTO.getPacsBillNeoDTO().getVitals().add(vitalBp2);
            // }
            // 测试数据结束

            System.out.println(billNeoMaxDTO);
            billNeoMaxDTOList.add(billNeoMaxDTO);
        }
        processRule(billNeoMaxDTOList, wordCrfDTO, res);
    }

    // 规则处理
    public void processRule(List<BillNeoMaxDTO> billNeoMaxDTOList, WordCrfDTO wordCrfDTO, IndicationDTO indicationDTO) {
        DiagLabel diagLabel = wordCrfDTO.getDiagLabel();
        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        PresentLabel presentLabel = wordCrfDTO.getPresentLabel();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();
        List<Medicine> allergyAll = new ArrayList<>();
        List<Medicine> takeMedicine = presentLabel.getTakeMedicine();
        List<Medicine> allergyDrug = new ArrayList<>();
        List<Drug> drug = wordCrfDTO.getDrug();
        for(Item item:drug){
            Medicine operaData = new Medicine();
            operaData.setName(item.getName());
            operaData.setStandName(item.getUniqueName());
            allergyDrug.add(operaData);
        }
        allergyAll.addAll(allergyDrug);
        allergyAll.addAll(takeMedicine);
        // 诊断数据
        List<Item> diags = wordCrfDTO.getDiag();

        //辅助检查描述
        PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
        List<Item> pacsDescList = new ArrayList<>();
        pacsLabel.getRes().stream().forEach(
                x -> {
                    Item item = new Item();
                    item.setName(x.getName());
                    item.setUniqueName(x.getUniqueName());
                    pacsDescList.add(item);
                }
        );
        // pacsLabel.getPacsNewList().stream().filter(x -> ListUtil.isNotEmpty(x.getPacsResults()) || ListUtil.isNotEmpty(x.getDisease())).forEach(
        //         x ->{
        //             List<String> pacsResults = x.getPacsResults().stream().map(pacr -> pacr.getStandName()).collect(Collectors.toList());
        //             List<String> diseases = x.getDisease().stream().map(disease -> disease.getStandName()).collect(Collectors.toList());
        //             pacsResults.addAll(diseases);
        //             pacsResults.stream().forEach(z ->{
        //                 Item item = new Item();
        //                 item.setName(z);
        //                 item.setUniqueName(z);
        //                 pacsDescList.add(item);
        //             });
        //         }
        // );

        // 手术数据
        List<Operation> operations_all = new ArrayList<>();
        List<Operation> operations_present = presentLabel.getOperations();
        List<Operation> operations_past = pastLabel.getOperations();
        List<Item> operation = wordCrfDTO.getOperation();
        List<Operation> operationsItem = new ArrayList<>();
        for(Item item:operation){
            Operation operaData = new Operation();
            operaData.setName(item.getName());
            operaData.setStandName(item.getUniqueName());
            operationsItem.add(operaData);
        }
        operations_all.addAll(operations_present);
        operations_all.addAll(operations_past);
        operations_all.addAll(operationsItem);

        // 临床表现数据
        List<Clinical> clinicals = new ArrayList<>();
        if(chiefLabel != null && ListUtil.isNotEmpty(chiefLabel.getClinicals())){
            clinicals.addAll(chiefLabel.getClinicals());
        }
        // 现病史临床表现数据先不用
        // if(presentLabel != null && ListUtil.isNotEmpty(presentLabel.getClinicals())){
        //     clinicals.addAll(presentLabel.getClinicals());
        // }

        List<BillMsg> billMsgList = new ArrayList<>();
        for (BillNeoMaxDTO bill : billNeoMaxDTOList) {
            // 性别
            sexRule.bill(wordCrfDTO, bill, billMsgList, NeoEnum.gender.getName());

            // 年龄
            ageRule.bill(wordCrfDTO, bill, billMsgList, NeoEnum.ageNeoDTO.getName());

            // 诊断
            commonRule.compareItemWithBill(bill.getDisease(), diags, bill, billMsgList, NeoEnum.disease.getName());

            // 化验
            lisRule.bill(wordCrfDTO.getLis(), bill, billMsgList, NeoEnum.lis.getName());

            // 体征
            vitalRule.bill(wordCrfDTO.getVitalLabel(), bill, billMsgList, NeoEnum.vitals.getName());

            // 辅检
            pacsRule.bill(bill.getPacs(), wordCrfDTO.getPacs(), bill, billMsgList, NeoEnum.pacs.getName());

            // 临床表现
            commonRule.compareNameWithBill(bill.getClinicfindings(), clinicals, bill, billMsgList, NeoEnum.clinicfindings.getName());

            // 手术(既往史、现病史)
            commonRule.compareNameWithBill(bill.getOperations(), operations_all, bill, billMsgList, NeoEnum.operations.getName());

            // 禁忌过敏药品（既往史）
            drugRule.bill(bill.getAllergicmeds(), pastLabel.getAllergyMedicines(), bill, billMsgList, NeoEnum.allergicmeds.getName());

            // 服用药品（现病史一般情况后的药品）
            drugRule.bill(bill.getOralmeds(), allergyAll, bill, billMsgList, NeoEnum.oralmeds.getName());

            // 禁用人群
            groupRule.bill(wordCrfDTO, bill, billMsgList, NeoEnum.group.getName());

            // 禁用辅助检查描述
            commonRule.compareItemWithBill(bill.getPacsDesc(), pacsDescList, bill, billMsgList, NeoEnum.pacsDesc.getName());
        }

        // 24小时重复开单项
        commonRule.repeat24Bill(wordCrfDTO, billMsgList);
        indicationDTO.setBillMsgList(billMsgList);
    }

}
