package com.diagbot.repository;


import com.diagbot.entity.node.*;
import com.diagbot.entity.node.base.BaseNode;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;

public interface BaseNodeRepository extends Neo4jRepository<BaseNode, Long> {

    List<BaseNode> findByNameIs(String name);

    @Query("MATCH (n) RETURN DISTINCT(LABELS(n))")
    List<String> getLabels();

    @Query("MATCH (n:实验室检查套餐) RETURN DISTINCT(KEYS(n))")
    List<List<String>> getLisSetKeys();

    @Query("MATCH (n:实验室检查) RETURN DISTINCT(KEYS(n))")
    List<List<String>> getLisNameKeys();

    @Query("MATCH (c:药品化学物质类别)<-[r:药品相关药品化学物质类别]-(y:药品代码通用名)-[r1:药品相关药品通用名称]->(m:药品通用名称) " +
            " RETURN DISTINCT(m.name)+'::'+c.name")
    List<String> getMedHuaXueClass();

    @Query("MATCH (c:药品治疗学类别)<-[r:药品相关药品治疗学类别]-(y:药品代码通用名)-[r1:药品相关药品通用名称]->(m:药品通用名称) " +
            " RETURN DISTINCT(m.name)+'::'+c.name")
    List<String> getMedZhiLiaoClass();

    @Query("MATCH (c:药品药理学类别)<-[r:药品相关药品药理学类别]-(y:药品代码通用名)-[r1:药品相关药品通用名称]->(m:药品通用名称) " +
            " RETURN DISTINCT(m.name)+'::'+c.name")
    List<String> getMedYaoLiClass();

    @Query("MATCH (c:药品解剖学类别)<-[r:药品相关药品解剖学类别]-(y:药品代码通用名)-[r1:药品相关药品通用名称]->(m:药品通用名称) " +
            " RETURN DISTINCT(m.name)+'::'+c.name")
    List<String> getMedJiePouClass();

    @Query("match(s) where any(label in labels(s) where label in ['症状','体征']) return distinct s.name")
    List<String> getSymptomClass();

    @Query("match(d:医保疾病名称)-[r:医保疾病名称相关性别]->(h) return DISTINCT d.name+'&'+h.name")
    List<String> getDisSexClass();

    @Query("match(d:医保疾病名称)-[r:医保疾病名称相关年龄]->(h) return DISTINCT d.name+'&'+h.name")
    List<String> getDisAgeClass();

    @Query("match(d:医保疾病名称)-[r:医保疾病名称相关发病率]->(h) return DISTINCT d.name+'&'+h.name")
    List<String> getDisdistributionClass();

    @Query("MATCH (n:实验室检查套餐名) WHERE EXISTS(n.拼音编码) AND n.拼音编码 CONTAINS {py} RETURN n")
    List<LisSet> getLisSetbyPinYin(@Param("py") String pycode);

    @Query("MATCH (n:实验室检查名称) WHERE EXISTS(n.拼音编码) AND n.拼音编码 CONTAINS {py} RETURN n")
    List<LisName> getLisbyPinYin(@Param("py") String pycode);

    @Query("MATCH (n:辅助检查名称) WHERE EXISTS(n.拼音编码) AND n.拼音编码 CONTAINS {py} RETURN n")
    List<PacsName> getPacsbyPinYin(@Param("py") String pycode);

    @Query("MATCH (n:医保疾病名称) WHERE EXISTS(n.拼音编码) AND n.拼音编码 CONTAINS {py} RETURN n")
    List<YiBaoDiseaseName> getYiBaoDiseasebyPinYin(@Param("py") String pycode);

    @Query("MATCH (n:医保手术与操作名称) WHERE EXISTS(n.拼音编码) AND n.拼音编码 CONTAINS {py} RETURN n")
    List<YiBaoOperationName> getYiBaoOperationbyPinYin(@Param("py") String pycode);

    @Query("MATCH (n:药品通用名称) WHERE EXISTS(n.拼音编码) AND n.拼音编码 CONTAINS {py} RETURN n")
    List<Medicine> getMedicinebyPinYin(@Param("py") String pycode);

    @Query("MATCH (n:科室) WHERE EXISTS(n.拼音编码) AND n.拼音编码 CONTAINS {py} RETURN n")
    List<Dept> getDeptbyPinYin(@Param("py") String pycode);

}