package com.diagbot.repository;

import com.diagbot.dto.*;
import com.diagbot.entity.node.*;
import com.diagbot.util.ListUtil;
import com.diagbot.util.NeoUtil;

import java.math.BigDecimal;
import java.util.*;

public class MedicineNode {


    public DrugBillNeoDTO MedicinetoDrugDTO(Medicine medicine) {
        DrugBillNeoDTO drugBillNeoDTO = new DrugBillNeoDTO();

        Set<Age> ages = medicine.getAge();
        for (Age age : ages) {
            NodeNeoDTO nodeNeoDTO = fillNodeNeoDTO(age);
            drugBillNeoDTO.setAgeNeoDTO(nodeNeoDTO);
        }

        Set<Group> groups = medicine.getGroup();
        for (Group group : groups) {
            NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            nodeNeoDTO.setName(group.getName());
            nodeNeoDTO.setTermtype(Constants.group);
            drugBillNeoDTO.getGroup().add(nodeNeoDTO);
        }


        Set<MedAllergen> medAllergens = medicine.getMedAllergens();
        for (MedAllergen medAllergen : medAllergens) {
            NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            nodeNeoDTO.setName(medAllergen.getName());
            nodeNeoDTO.setTermtype(Constants.yaoping);
            drugBillNeoDTO.getAllergicmeds().add(nodeNeoDTO);
        }


        Set<Disease> diseases = medicine.getDisease();
        for (Disease disease : diseases) {
            NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            nodeNeoDTO.setName(disease.getName());
            nodeNeoDTO.setTermtype(Constants.disease);
            drugBillNeoDTO.getDisease().add(nodeNeoDTO);
        }

        return drugBillNeoDTO;
    }


    public NodeNeoDTO fillNodeNeoDTO(Age age) {
        NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
        Integer range;
        BigDecimal max, min;

        nodeNeoDTO.setName(age.getName());

        max = (age.getMaxval()==null)?null:new BigDecimal(age.getMaxval());
        nodeNeoDTO.setMax(max);

        min = (age.getMinval()==null)?null:new BigDecimal(age.getMinval());
        nodeNeoDTO.setMin(min);

        range = (age.getRange()==null)?0:age.getRange();
        nodeNeoDTO.setValType(range);

        nodeNeoDTO.setName(age.getTermname());

        return nodeNeoDTO;
    }



    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getMedicineBill(Map<String, String> medsterm, MedicineRepository medicineRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        DrugBillNeoDTO drugBillNeoDTO;

        List<Medicine> medicine;
        String medname;

        for (String term : medsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            medname = medsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(medname);

            drugBillNeoDTO = new DrugBillNeoDTO();

            medicine = medicineRepository.findByNameIs(medname);

            if (medicine != null && medicine.size() > 0) {
                drugBillNeoDTO = MedicinetoDrugDTO(medicine.get(0));

                billNeoDTO.setDrugBillNeoDTO(drugBillNeoDTO);

                billNeoDTOs.add(billNeoDTO);
            }
        }


        return billNeoDTOs;
    }



    /**
     * 获取药品商品名称列表
     */
    public List<DrugInfoDTO> getMedicineList(String name, MedicineRepository medicineRepository, BaseNodeRepository baseNodeRepository) {
        String nodename;

        List<DrugInfoDTO> medslist = new ArrayList<>();
        DrugInfoDTO drugInfoDTO;

        List<Medicine> medicines = medicineRepository.findByNameContaining(name);
        medicines.addAll(baseNodeRepository.getMedicinebyPinYin(name));

        if (ListUtil.isNotEmpty(medicines) && medicines.size() > 100) {
            medicines = medicines.subList(0, 100);
        }

        for (Medicine med : medicines) {
            drugInfoDTO = new DrugInfoDTO();
            nodename = med.getName();

            drugInfoDTO.setName(nodename);
            medslist.add(drugInfoDTO);
        }


        return medslist;
    }
}

