package com.diagbot.util;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.model.entity.Clinical;
import com.diagbot.model.entity.Usual;
import com.diagbot.model.entity.Vital;
import com.diagbot.model.label.VitalLabel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @description:
 * @author: zhoutg
 * @time: 2020/7/30 15:18
 */
@Slf4j
@Component
public class CoreUtil {

    public static Boolean debugFlag;

    @Value("${debugFlag}")
    public void setDebugFlag(Boolean debugFlag) {
        CoreUtil.debugFlag = debugFlag;
    }

    /**
     * 将列表中属性对应的值以list形式返回
     *
     * @param list
     * @param propertyName
     * @param <T>
     * @return
     */
    public static <T> List<String> getPropertyList(List<T> list, String propertyName) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                String val = (String)getFieldValue(t, propertyName);
                if (StringUtil.isNotBlank(val)) {
                    res.add(val);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    /**
     * 将列表中属性(name)对应的值以list形式返回
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> getPropertyList(List<T> list) {
        return getPropertyList(list, "name");
    }

    /**
     * 将列表中属性对应的值从map中获取，如果有值，将standName替换
     *
     * @param list
     * @param propertyName
     * @param <T>
     * @return
     */
    public static <T> List<String> setPropertyList(List<T> list, String propertyName, String standName, Map<String, String> map) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                setFieldValue(t, propertyName, standName, map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    public static <T> void  setPropertyList(T list, String propertyName, String standName, Map<String, String> map) {
        if(list == null){
            return;
        }
        try {
            setFieldValue(list, propertyName, standName, map);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 将列表中属性对应的值从map中获取，如果有值，将standName替换
     *
     * @param list
     * @param <T>
     * @return
     */
    public static <T> List<String> setPropertyList(List<T> list, Map<String, String> map) {
        return setPropertyList(list, "name", "standName", map);
    }


    /**
     * 循环向上转型, 获取对象的 DeclaredField
     * @param object : 子类对象
     * @param fieldName : 父类中的属性名
     * @return 父类中的属性对象
     */
    public static Field getDeclaredField(Object object, String fieldName){
        Field field = null ;

        Class<?> clazz = object.getClass() ;

        for(; clazz != Object.class ; clazz = clazz.getSuperclass()) {
            try {
                field = clazz.getDeclaredField(fieldName) ;
                return field ;
            } catch (Exception e) {
                //这里甚么都不要做！并且这里的异常必须这样写，不能抛出去。
                //如果这里的异常打印或者往外抛，则就不会执行clazz = clazz.getSuperclass(),最后就不会进入到父类中了
            }
        }

        return null;
    }

    /**
     * 直接读取对象的属性值, 忽略 private/protected 修饰符, 也不经过 getter
     * @param object : 子类对象
     * @param fieldName : 父类中的属性名
     * @return : 父类中的属性值
     */
    public static Object getFieldValue(Object object, String fieldName){

        //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
        Field field = getDeclaredField(object, fieldName) ;

        //抑制Java对其的检查
        field.setAccessible(true) ;

        try {
            //获取 object 中 field 所代表的属性值
            return field.get(object) ;

        } catch(Exception e) {
            e.printStackTrace() ;
        }

        return null;
    }

    /**
     * 直接设置对象属性值, 忽略 private/protected 修饰符, 也不经过 setter
     * @param object : 子类对象
     * @param name : 父类中的属性名
     * @param standName : 要替换的属性名
     * @param value : 将要设置的值
     */
    public static void setFieldValue(Object object, String name, String standName, Map<String, String> value){

        //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
        Field field = getDeclaredField(object, name) ;

        //抑制Java对其的检查
        field.setAccessible(true) ;

        try {
            //将 object 中 field 所代表的值 设置为 value
            String key = (String)field.get(object);
            if (value != null && value.get(key) != null) {
                Field standField = getDeclaredField(object, standName) ;
                standField.setAccessible(true) ;
                standField.set(object, value.get(key));
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

    }

    public static <T> List<String> setPropertyList(List<T> list, String name, String detailName,String uniqueName, Map<String, String> map) {
        List<String> res = new ArrayList<>();
        if (ListUtil.isEmpty(list)) {
            return res;
        }
        for (T t : list) {
            try {
                setFieldValue(t, name, detailName,uniqueName, map);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res;
    }

    public static void setFieldValue(Object object, String name, String standName,String unique, Map<String, String> value){

        //根据 对象和属性名通过反射 调用上面的方法获取 Field对象
        Field field = getDeclaredField(object, name) ;
        Field field1 = getDeclaredField(object, standName) ;
        //抑制Java对其的检查
        field.setAccessible(true) ;
        field1.setAccessible(true) ;

        try {
            //将 object 中 field 所代表的值 设置为 value
            String key = (String)field.get(object);
            String key1 = (String)field1.get(object);
            String lis_c = key+key1;
            if (value != null && value.get(lis_c) != null) {
                Field standField = getDeclaredField(object, unique) ;
                standField.setAccessible(true) ;
                standField.set(object, value.get(lis_c));
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

    }

    /**
     * 将对象添加到列表中
     *
     * @param list
     * @param t
     * @param <T>
     */
    public static <T> void addBeanToList(List<T> list, T t) {
        if (list == null) {
            list = new ArrayList<T>();
        }
        if (t != null) {
            list.add(t);
        }
    }

    /**
     * 比较name是否一样
     *
     * @param nodeNeoDTO
     * @param item
     * @return
     */
    public static Boolean compareNameWithNodeNeoDTO(NodeNeoDTO nodeNeoDTO,  List<? extends Item> item) {
        if (ListUtil.isEmpty(item) || nodeNeoDTO == null) {
            return false;
        }
        for (Item it : item) {
            if (it.getUniqueName().equals(nodeNeoDTO.getName())) {
                return true;
            }
        }
        return  false;
    }

    /**
     * 比较化验是否匹配
     *
     * @param nodeNeoDTO
     * @param lis
     * @return
     */
    public static Map<String, Object> compareLis(NodeNeoDTO nodeNeoDTO, Lis lis) {
        Map<String, Object> map = new LinkedHashMap<>();
        boolean flag = false;
        if (nodeNeoDTO == null || lis == null) {
            map.put("flag", flag);
            return map;
        }
        if (StringUtil.isNotBlank(lis.getOtherValue())) {
            if (lis.getOtherValue().equals(nodeNeoDTO.getVal())) {
                map.put("msg", lis.getName() + lis.getDetailName() + lis.getOtherValue());
                flag = true;
            }
        } else if (lis.getValue() != null) {
            double value = lis.getValue();
            flag = compareNum(nodeNeoDTO, value);
            if (flag) {
                map.put("msg", lis.getName() + lis.getDetailName() + subZeroAndDot(String.valueOf(lis.getValue())));
            }
        }
        map.put("flag", flag);
        return map;
    }


    /**
     * 比较查体是否匹配
     *
     * @param nodeNeoDTO
     * @param vitalLabel
     * @return
     */
    public static Map<String, Object> compareVital(NodeNeoDTO nodeNeoDTO, VitalLabel vitalLabel) {
        Map<String, Object> map = new LinkedHashMap<>();
        List<Vital> vitalList = vitalLabel.getVitals(); // 体征数据
        List<Clinical> clinicals = vitalLabel.getClinicals(); // 体征临床表现
        boolean flag = false;
        if (nodeNeoDTO == null) {
            map.put("flag", flag);
            return map;
        }
        if (nodeNeoDTO.getMax() == null && nodeNeoDTO.getMin() == null) {
            // 体征名称比较(例如：喉头水肿)
            for (Clinical clinical : clinicals) {
                if (nodeNeoDTO.getName().contains(clinical.getStandName())) {
                    map.put("msg", clinical.getName());
                    flag = true;
                }
            }
        } else {
            // 体征数值比较(例如：体温，血压)
            for (Vital vital : vitalList) {
                List<Usual> usualList = vital.getUsualList();
                if (nodeNeoDTO.getName().equals(vital.getStandName()) && vital.getPd() != null
                        && StringUtil.isNotBlank(vital.getPd().getValue())) {
                    flag = compareNum(nodeNeoDTO, Double.parseDouble(vital.getPd().getValue()));
                    if (flag) {
                        map.put("msg", vital.getName() + subZeroAndDot(String.valueOf(vital.getPd().getValue())));
                    }
                } else if (ListUtil.isNotEmpty(usualList)) {
                    for (Usual usual : usualList) {
                        if (nodeNeoDTO.getName().equals(usual.getStandName())) {
                            flag = compareNum(nodeNeoDTO, Double.parseDouble(usual.getValue()));
                            if (flag) {
                                map.put("msg", vital.getName() + subZeroAndDot(String.valueOf(vital.getPd().getValue())));
                                break;
                            }
                        }
                    }
                }
            }
        }

        map.put("flag", flag);
        return map;
    }

    /**
     * 比较数值是否匹配
     *
     * @param nodeNeoDTO
     * @param name
     * @param standName
     * @param value
     * @return
     */
    public static Map<String, Object> compareNumWtihMap(NodeNeoDTO nodeNeoDTO, String name, String standName, String value) {
        Map<String, Object> map = new LinkedHashMap<>();
        boolean flag = false;
        if (nodeNeoDTO == null || StringUtil.isBlank(name) || StringUtil.isBlank(standName) || StringUtil.isBlank(value)) {
            map.put("flag", flag);
            return map;
        }
        try {
            flag = compareNum(nodeNeoDTO, Double.parseDouble(value));
            if (flag) {
                map.put("msg", name + subZeroAndDot(String.valueOf(value)));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        map.put("flag", flag);
        return map;
    }

    /**
     * 比较数值大小（int）
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static boolean compareNum(NodeNeoDTO nodeNeoDTO, int value) {
        boolean flag = false;
        BigDecimal min = nodeNeoDTO.getMin();
        BigDecimal max = nodeNeoDTO.getMax();

        // 取区间
        if (nodeNeoDTO.getValType() == 0) {
            if (min != null && max != null) {
                int minNeo = min.intValue();
                int maxNeo = max.intValue();
                if (minNeo <= value && value <= maxNeo) {
                    flag = true;
                }
            } else if (min != null && max == null) {
                int minNeo = min.intValue();
                if (minNeo <= value) {
                    flag = true;
                }
            } else if (min == null && max != null) {
                int maxNeo = max.intValue();
                if (value <= maxNeo) {
                    flag = true;
                }
            }
        } else {
            // 取两头
            if (min != null) {
                int minNeo = min.intValue();
                if (value <= minNeo) {
                    flag = true;
                }
            }
            // 取两头
            if (max != null) {
                int maxNeo = max.intValue();
                if (value >= maxNeo) {
                    flag = true;
                }
            }
        }
        return flag;
    }

    /**
     * 比较数值大小（double）
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static boolean compareNum(NodeNeoDTO nodeNeoDTO, double value) {
        boolean flag = false;
        BigDecimal min = nodeNeoDTO.getMin();
        BigDecimal max = nodeNeoDTO.getMax();

        // 取区间
        if (nodeNeoDTO.getValType() == 0) {
            if (min != null && max != null) {
                double minNeo = min.doubleValue();
                double maxNeo = max.doubleValue();
                if (minNeo <= value && value <= maxNeo) {
                    flag = true;
                }
            } else if (min != null && max == null) {
                double minNeo = min.doubleValue();
                if (minNeo <= value) {
                    flag = true;
                }
            } else if (min == null && max != null) {
                double maxNeo = max.doubleValue();
                if (value <= maxNeo) {
                    flag = true;
                }
            }
        } else {
            // 取两头
            if (min != null) {
                double minNeo = min.doubleValue();
                if (value <= minNeo) {
                    flag = true;
                }
            }
            // 取两头
            if (max != null) {
                double maxNeo = max.doubleValue();
                if (value >= maxNeo) {
                    flag = true;
                }
            }
        }
        return flag;
    }

    /**
     * 获取提示信息（double）
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static String getNumMsg(NodeNeoDTO nodeNeoDTO, String name, double value, String unit) {
        if (nodeNeoDTO == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        sb.append(name);
        sb.append(subZeroAndDot(String.valueOf(value))); // 去掉小数点后的0
        if (StringUtil.isBlank(unit)) {
            sb.append(unit);
        }
        sb.append("(禁忌范围[");
        if (nodeNeoDTO.getMin() != null) {
            sb.append(nodeNeoDTO.getMin());
        }
        sb.append(",");
        if (nodeNeoDTO.getMax() != null) {
            sb.append(nodeNeoDTO.getMax());
        }
        // if (nodeNeoDTO.getUnit() != null) {
        //     sb.append(nodeNeoDTO.getUnit());
        // }
        sb.append("])");
        return sb.toString();
    }

    public static String subZeroAndDot(String s){
        if(s.indexOf(".") > 0){
            s = s.replaceAll("0+?$", "");//去掉多余的0
            s = s.replaceAll("[.]$", "");//如最后一位是.则去掉
        }
        return s;
    }

    /**
     * 获取提示信息（例如：年龄19[0,20]）
     *
     * @param nodeNeoDTO
     * @return
     */
    public static String getStrMsgWithRange(NodeNeoDTO nodeNeoDTO, String name, String value) {
        if (nodeNeoDTO == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        sb.append(name).append(value).append("[");
        if (nodeNeoDTO.getMin() != null) {
            sb.append(nodeNeoDTO.getMin());
        }
        sb.append(",");
        if (nodeNeoDTO.getMax() != null) {
            sb.append(nodeNeoDTO.getMax());
        }
        sb.append("]");
        return sb.toString();
    }

    /**
     * 获取提示信息（String）
     *
     * @param nodeNeoDTO
     * @param value
     * @return
     */
    public static String getStrMsg(NodeNeoDTO nodeNeoDTO, String name, String value) {
        if (nodeNeoDTO == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        sb.append(name).append(value);
        return sb.toString();
    }

    /**
     * list 转 string
     * @param list
     * @return
     */
    public static  String listConvertString(List<String> list){
        return StringUtils.join(list, ",");
    }

    /**
     * 输出调试信息
     *
     * @param start
     * @param msg
     * @param debugMap
     */
    public static void getDebugStr(long start, String msg, Map<String, Object> debugMap) {
        long end = System.currentTimeMillis();
        debugMap.put(msg,  + (end - start) / 1000.0 + "秒");
    }

    /**
     * 输出调试信息
     *
     * @param msg
     * @param debugMap
     */
    public static void getDebugObject(String msg, Map<String, Object> debugMap, Object obj) {
        if (debugFlag) {
            debugMap.put(msg, obj);
        }
    }

    /**
     * 返回化验的结果
     *
     * @param lis
     * @return
     */
    public static String getLisValue(Lis lis) {
        if (lis == null) {
            return "";
        }
        return StringUtil.isNotBlank(lis.getOtherValue()) ? lis.getOtherValue() : subZeroAndDot(lis.getValue().toString());
    }

    /**
     * 输出调试信息
     *
     * @param msg
     * @param error
     * @param debugMap
     */
    public static void getDebugStr(String msg, Object error, Map<String, Object> debugMap) {
        debugMap.put(msg,  error);
    }
}
